<?php
/* vim: set expandtab tabstop=4 softtabstop=4 shiftwidth=4:
  Codificación: UTF-8
  +----------------------------------------------------------------------+
  | Elastix version 2.4.0-1                                              |
  | http://www.elastix.org                                               |
  +----------------------------------------------------------------------+
  | Copyright (c) 2006 Palosanto Solutions S. A.                         |
  +----------------------------------------------------------------------+
  | Cdla. Nueva Kennedy Calle E 222 y 9na. Este                          |
  | Telfs. 2283-268, 2294-440, 2284-356                                  |
  | Guayaquil - Ecuador                                                  |
  | http://www.palosanto.com                                             |
  +----------------------------------------------------------------------+
  | The contents of this file are subject to the General Public License  |
  | (GPL) Version 2 (the "License"); you may not use this file except in |
  | compliance with the License. You may obtain a copy of the License at |
  | http://www.opensource.org/licenses/gpl-license.php                   |
  |                                                                      |
  | Software distributed under the License is distributed on an "AS IS"  |
  | basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See  |
  | the License for the specific language governing rights and           |
  | limitations under the License.                                       |
  +----------------------------------------------------------------------+
  | The Original Code is: Elastix Open Source.                           |
  | The Initial Developer of the Original Code is PaloSanto Solutions    |
  +----------------------------------------------------------------------+
  $Id: paloSantoOpenVPN.class.php,v 1.1 2013-06-10 07:10:30 Nikita Rukavkov admin@voip-lab.ru Exp $ */

include_once("libs/paloSantoDB.class.php");
include_once("libs/paloSantoConfig.class.php");
include_once("libs/paloSantoNetwork.class.php");
include_once("Telnet.class.php");

/* Class OpenVPN Server */
class PaloSantoOpenVPN
{
    var $_DB;
    var $errMsg;
    var $oConfig;
    var $filename_cfg;
    var $rsa_path;
    var $mConfig;
    var $conn;

    function PaloSantoOpenVPN(&$pDB)
    {
        if (is_object($pDB)) {
	    $this->_DB =& $pDB;
    	    $this->errMsg = $this->_DB->errMsg;
        } else {
    	    $dsn = (string)$pDB;
            $this->_DB = new paloDB($dsn);
            if (!$this->_DB->connStatus) {
    	        $this->errMsg = $this->_DB->errMsg;
            } 
        }
        $this->mConfig = array();
        $this->mConfig['pidpath'] = "/var/run/openvpn/server.pid";
	$this->rsa_path="/etc/openvpn/easy-rsa/keys/";
	try
	{
	    $this->conn = new Telnet("127.0.0.1", 7000, 1, "");
	    $this->conn->clearBuffer();
	}
	catch (Exception $e)
	{
	    $this->errMsg=$e->getMessage();
	}
	
    }
    
    function disconnectUser($client)
    {
	if($this->getStatusServiceOpenVPN() == 'active')
	{
	    try
	    {
		$this->conn->exec("kill $client");
		sleep(2);
	    }
	    catch (Exception $e)
	    {
	    }
	}
    }
    
    function loadExportVars($name)
    {
	if($name!='')
	{
	    $ret = $output = NULL;
	    exec ("egrep '$name' /etc/openvpn/easy-rsa/vars | awk 'BEGIN { FS = \"=\" } ; { print $2 }'",$output,$ret);
	    return str_replace('"',"",trim($output[0]));
	}
	else return;
    }
    
    function addUserOpenVPN($exportVars)
    {
	foreach($exportVars as &$value)
    	    if($value=='') $value='%EMPTY%';
    	

        $sComando = '/usr/bin/elastix-helper openvpn_server --adduser'.
                    ' --common_name '.escapeshellarg($exportVars['common_name']).
                    ' --email '.escapeshellarg($exportVars['email']).
                    ' --country '.escapeshellarg($exportVars['country']).
            	    ' --province '.escapeshellarg($exportVars['province']).
                    ' --city '.escapeshellarg($exportVars['city']).
                    ' --organization '.escapeshellarg($exportVars['organization']).
                    ' --cn '.escapeshellarg($exportVars['cn']).
                    ' --staticip '.escapeshellarg($exportVars['static_ip']).
                    ' --expdate '.escapeshellarg($exportVars['expdate']).
                    ' 2>&1';
        $output = $ret = NULL;
        
	exec($sComando, $output, $ret);
        if ($ret != 0) {
                $this->errMsg = implode('', $output);
		return FALSE;
		
        }
        else
        {
            //$this->pConfig->escribir_configuracion($arrValues);
            return TRUE;
        }
	
    }

    function getStaticIpListOpenVPN($filter = NULL,$common_name = NULL)
    {
	$arrSettings = $this->loadSettings();
	$server_network = $arrSettings['pa_server_network'];

        $sComando = '/usr/bin/elastix-helper openvpn_server --liststaticip'.
                    ' --server_network '.escapeshellarg($server_network).
                    (isset($common_name) ? " --common_name ".escapeshellarg($common_name) : "").
                    ' 2>&1';
        $output = $ret = NULL;
        

	exec($sComando, $output, $ret);
	
	array_unshift($output,"");
	
	$static_list = array();
	
	foreach($output as $item)
	{
	    if(isset($filter))
	    {
		if($filter != '')
		{
		    if(strpos($item,$filter.'-') !== FALSE)
		    {
			return $item;
		    }
		}
		else return '';
	    }
	    else
	    {
		$static_list[$item] = $item;
	    }
	}
	return $static_list;
	
    }
    function editUserOpenVPN($arrParams)
    {
	foreach($arrParams as &$value)
    	    if($value=='') $value='%EMPTY%';

        $sComando = '/usr/bin/elastix-helper openvpn_server --edituser'.
                    ' --common_name '.escapeshellarg($arrParams['common_name']).
                    ' --staticip '.escapeshellarg($arrParams['static_ip']).
                    ' 2>&1';
                
    
	$output = $ret = NULL;
	exec($sComando, $output, $ret);
	
	$this->disconnectUser($arrParams['common_name']);

	return TRUE;
    }
    
    function addServerOpenVPN($exportVars)
    {

	foreach($exportVars as &$value)
    	    if($value=='') $value='%EMPTY%';
        $sComando = '/usr/bin/elastix-helper openvpn_server --addserver'.
                    ' --email '.escapeshellarg($exportVars['email']).
                    ' --country '.escapeshellarg($exportVars['country']).
                    ' --province '.escapeshellarg($exportVars['province']).
                    ' --city '.escapeshellarg($exportVars['city']).
                    ' --organization '.escapeshellarg($exportVars['organization']).
                    ' --cn '.escapeshellarg($exportVars['cn']).
                    ' 2>&1';
        
    

	//UPDATE COMPANY SET ADDRESS = 'Texas', SALARY = 20000.00;
/*        $sComando = 'UPDATE settings SET '.
                    ' --email '.escapeshellarg($exportVars['email']).
                    ' --country '.escapeshellarg($exportVars['country']).
                    ' --province '.escapeshellarg($exportVars['province']).
                    ' --city '.escapeshellarg($exportVars['city']).
                    ' --organization '.escapeshellarg($exportVars['organization']).
                    ' --cn '.escapeshellarg($exportVars['cn']).
                    ' 2>&1';
*/                    
                    
        $output = $ret = NULL;
	exec($sComando, $output, $ret);
	
	$ret_entry = split("\n", `/usr/bin/elastix-helper openvpn_server --dumpkeys --dir $this->rsa_path`);
	foreach($ret_entry as $n => $tmp_entry)
	{
	    $val = split(".crt",$tmp_entry);
	    if($val[0]!='') $this->disconnectUser($val[0]);
	    
	}
	
        if ($ret != 0) {
                $this->errMsg = implode('', $output);
		return FALSE;
        }
        else
        {
            return TRUE;
        }
	
    }

    //Function adding (!) symbol if ip in not network
    function genIP($v_ip)
    {
	global $arrLang;
        $settings = $this->loadSettings();
    	if($v_ip != "")
	{
	    if(!$this->ip_in_range($v_ip,$settings['pa_server_network']))
	    $virtual_ip = "$v_ip <a href=#><img height=15px src='/modules/openvpn_server/images/alert.png' alt='".$arrLang['Invalid IP Address']."' title='".$arrLang['Invalid IP Address']."'></a>";
	    else $virtual_ip = $v_ip;
	}
	else $virtual_ip = '-';
	return $virtual_ip;
    }
    
    function load_openvpn_table($filter_common_name,$filter_email,$filter_real_ip,$filter_virtual_ip)
    {
	global $arrLang;
    	$rTable=array();
	$ret = split("\n", `/usr/bin/elastix-helper openvpn_server --dumpstatus`);
	$ret_entry = split("\n", `/usr/bin/elastix-helper openvpn_server --dumpkeys --dir $this->rsa_path`);
	
	foreach($ret_entry as $n => $tmp_entry)
	{
	    if($tmp_entry == "") continue;
	    $t_e=split(" ",$tmp_entry);
	    $entry = $t_e[0];
	    $entry_mail = $t_e[1];
	    $entry_expdate = $t_e[2];
	    $virtual_ip = $t_e[3];
	    
	    //Добавляем алерт для заканчивающихся сертификатов
	    //1 месяц - рыжий
	    //Если закончился - красный
	    
	    $difftime = (strtotime($entry_expdate) - strtotime(date("d.m.Y")))/3600/24;
	    if ($difftime <= 30 && $difftime>0)
	        $entry_expdate.=" <a href=#><img height=15px src='/modules/openvpn_server/images/warning.png' alt='".$arrLang['Certificate is coming to an end']."' title='".$arrLang['Certificate is coming to an end']."'></a>";
	    elseif($difftime <= 0)
		$entry_expdate.=" <a href=#><img height=15px src='/modules/openvpn_server/images/alert.png' alt='".$arrLang['Certificate is not valid']."' title='".$arrLang['Certificate is not valid']."'></a>";
		
	    $is_connected=FALSE;
	    //Check Client Connection
	    foreach($ret as $n => $rstr) {
		list($HEAD,$COMMON_NAME,$REAL_ADDRESS,$VIRTUAL_ADDRESS,$BYTES_RECIVED,$BYTES_SENT,$CONNECTED_SINCE,$CONNECTED_STATE,$USERNAME)=split(",",$rstr);
		$COMMON_NAME_FILE = str_replace(".crt","",$entry);
		
		if($HEAD == "CLIENT_LIST")
		{
		    if($COMMON_NAME == $COMMON_NAME_FILE)
		    {
			$rTable[] = array("<input type='checkbox' name='client[]' value='$COMMON_NAME' />","<a href='?menu=openvpn_server&edit_user=1&common_name=$COMMON_NAME&static_ip=".($VIRTUAL_ADDRESS!='' ? $VIRTUAL_ADDRESS : "")."'>$COMMON_NAME</a>",$entry_mail,$VIRTUAL_ADDRESS,$REAL_ADDRESS,round($BYTES_SENT/(1024*1024),2),round($BYTES_RECIVED/(1024*1024),2),$CONNECTED_SINCE,$entry_expdate,"<a href=\"?menu=openvpn_server&download=config&name=$COMMON_NAME\"><img alt=\"Download config\" width=\"20px\" src=\"modules/openvpn_server/images/save_btn.png\" /></a>&nbsp;<a href=\"?menu=openvpn_server&download=mail&name=$COMMON_NAME\"><img alt=\"Send config to user Email\" width=\"20px\" src=\"modules/openvpn_server/images/mail_btn.png\" /></a>");
			$is_connected = TRUE;
			break;
		    }
		}
	    }
	    //If Client not connected
	    if(!$is_connected)
	    {
		$rTable[] = array("<input type='checkbox' name='client[]' value='$COMMON_NAME_FILE' />","<a href='?menu=openvpn_server&edit_user=1&common_name=$COMMON_NAME_FILE&static_ip=".($virtual_ip!='' ? $virtual_ip : "")."'>$COMMON_NAME_FILE</a>",$entry_mail,$this->genIP($virtual_ip),"-","-","-","-",$entry_expdate,"<a href=\"?menu=openvpn_server&download=config&name=$COMMON_NAME_FILE\"><img alt=\"Download config\" width=\"20px\" src=\"modules/openvpn_server/images/save_btn.png\" /></a>&nbsp;<a href=\"?menu=openvpn_server&download=mail&name=$COMMON_NAME_FILE\"><img alt=\"\Send config to user Email\" width=\"20px\" src=\"modules/openvpn_server/images/mail_btn.png\" /></a>");
	    }
	    //Apply Filters
	    if($filter_common_name != '')
	    {
		$f_name=$rTable[count($rTable)-1];
		if(strpos($f_name[1],$filter_common_name) === FALSE) { array_pop(&$rTable); continue; }
	    }

	    if($filter_email != '')
	    {
		$f_name=$rTable[count($rTable)-1];
		if(strpos($f_name[2],$filter_email) === FALSE) { array_pop(&$rTable); continue; }
	    }
	    
	    if($filter_real_ip != '')
	    {
		$f_name=$rTable[count($rTable)-1];
		if(strpos($f_name[4],$filter_real_ip) === FALSE) { array_pop(&$rTable); continue; }
	    }
	    
	    if($filter_virtual_ip != '')
	    {
		$f_name=$rTable[count($rTable)-1];
		if(strpos($f_name[3],$filter_virtual_ip) === FALSE) { array_pop(&$rTable); continue; }
	    }
	}
	
	return $rTable;
    }


    function getConfigurationOpenVPN() 
    {
        global $arrLang;
    }

    function getStatusServiceOpenVPN()
    {
        $output = $ret = NULL;
        exec("ps ax | grep '/var/run/openvpn/server.pid --cd /etc/openvpn --config server.conf' | grep -v grep", $output, $ret);
        return (count($output) >=1) ? 'active' : 'desactive';
            
    }

    function startServiceOpenVPN()
    {
        $this->errMsg = '';
        $settings = $this->loadSettings();
        
        if(!file_exists('/etc/openvpn/server.crt') || !file_exists('/etc/openvpn/server.key'))
        {
    	    $this->errMsg = 'CERT_ERR';
    	    return FALSE;
        }
        elseif( $settings['pa_local'] == '')
        {
	    $this->errMsg = 'SETTINGS_ERR';
    	    return FALSE;
    	}
    	else
    	{
    	    $sComando = '/usr/bin/elastix-helper openvpn_server --start 2>&1';
    	    $output = $ret = NULL;
    	    exec($sComando, $output, $ret);
    	    if ($ret != 0) {
        	$this->errMsg = implode('', $output);
        	return FALSE;
    	    }
    	    return TRUE;
    	}
    }

    function stopServiceOpenVPN()
    {
        $this->errMsg = '';
        $sComando = 'sudo /usr/sbin/elastix-helper openvpn_server --stop 2>&1';
        $output = $ret = NULL;
        exec($sComando, $output, $ret);
        if ($ret != 0) {
            $this->errMsg = implode('', $output);
            return FALSE;
        }
        return TRUE;
    }

    function downloadLogs()
    {
        $output = $ret = NULL;
        $sComando = '/usr/bin/elastix-helper openvpn_server --ziplogs 2>&1';	                            
        exec($sComando, $output, $ret);
        
        if($ret != 0) return FALSE;
        else return TRUE;
    }

    function updateFileConfOpenVPN($arrValues)
    {
	global $arrLang;
        $this->errMsg = '';
        foreach($arrValues as &$value)
    	    if($value=='') $value='%EMPTY%';
    	
        if($pingrestart < $pingcheck) 
        {
    	    $this->errMsg = 'Ping Every must be less then Restart After';
    	    return FALSE;
    	}
        
	$sComando = '/usr/bin/elastix-helper openvpn_server --config'.
                    ' --local '.escapeshellarg($arrValues['pa_local']).
                    ' --port '.escapeshellarg($arrValues['pa_port']).
                    ' --proto '.escapeshellarg($arrValues['pa_proto']).
                    ' --server_network '.escapeshellarg($arrValues['pa_server_network']).
                    ' --dns1 '.escapeshellarg($arrValues['pa_dns1']).
                    ' --dns2 '.escapeshellarg($arrValues['pa_dns2']).
                    ' --redirect_gw '.escapeshellarg($arrValues['pa_redirect_gw']).
                    ' --serveronly '.escapeshellarg($arrValues['pa_serveronly']).
                    ' --compr '.escapeshellarg($arrValues['pa_compr']).
                    ' --maxclients '.escapeshellarg($arrValues['pa_maxclients']).
                    ' --verbose '.escapeshellarg($arrValues['pa_verbose']).
                    ' --pingcheck '.escapeshellarg($arrValues['pa_ping']).
                    ' --pingrestart '.escapeshellarg($arrValues['pa_ping_restart']).
                    ' --enablenat '.escapeshellarg($arrValues['enable_nat']).
                    ' --static_routes '.escapeshellarg($arrValues['static_routes']).
                    ' 2>&1'; 
        
        $output = $ret = NULL;
        exec($sComando, $output, $ret);
        if ($ret != 0) {
            $this->errMsg = implode('', $output);
	    return FALSE;
        }
        else
        {
    	    $arrParameters = array( $arrValues['pa_local'],
    				    $arrValues['pa_port'],
    				    $arrValues['pa_proto'],
    				    $arrValues['pa_server_network'],
    				    $arrValues['pa_dns1'],
    				    $arrValues['pa_dns2'],
    				    $arrValues['pa_redirect_gw'],
    				    $arrValues['pa_serveronly'],
    				    $arrValues['pa_compr'],
    				    $arrValues['pa_maxclients'],
    				    $arrValues['pa_verbose'],
    				    $arrValues['pa_ping'],
    				    $arrValues['pa_ping_restart'],
    				    $arrValues['use_hostname'],
    				    $arrValues['enable_nat'],
    				    $arrValues['static_routes'] );
    				    
    				    
    	    $request = "UPDATE settings SET".
                " pa_local=?,".
                " pa_port=?,".
                " pa_proto=?,".
                " pa_server_network=?,".
                " pa_dns1=?,".
                " pa_dns2=?,".
                " pa_redirect_gw=?,".
                " pa_serveronly=?,".
                " pa_compr=?,".
                " pa_maxclients=?,".
                " pa_verbose=?,".
                " pa_ping=?,".
                " pa_ping_restart=?,".
                " use_hostname=?,".
                " enable_nat=?,".
                " static_routes=?";
            
    	    if(!$this->_DB->genQuery($request,$arrParameters))
    	    {
    		$this->errMsg = $this->_DB->errMsg;
    		return FALSE;
    	    }
	    else 
		return TRUE;
        }
    }
    
    function removeFromOpenVPN($clients)
    {
	foreach($clients as $client)
	{
	    exec("/usr/bin/elastix-helper openvpn_server --removeuser --client $client");
	    
	    $this->disconnectUser($client);
	}
    }


    function moveLicense($file_lic)
    {
	if(file_exists($file_lic))
    	{
    	    exec("/usr/bin/elastix-helper openvpn_server --movelicense --filelic ".escapeshellarg($file_lic),$output,$ret);
    	    if(file_exists('/var/www/html/modules/openvpn_server/configs/openvpn.lic')) return TRUE;
    		else return FALSE;
    	}
    	else
    	    return FALSE;
    }
    
    function checkLic()
    {
	$ret = $out = NULL;
	exec("/usr/bin/elastix-helper openvpn_server --checklic",$out,$ret);
	return $ret;
    }

    function getInterfacesOpenVPN($ifaces)
    {
	$arrResult = array();
	$arrResult['0.0.0.0'] = '0.0.0.0';
	foreach ($ifaces as $iface)
	    if($iface['Inet Addr']!='')
		$arrResult[$iface['Inet Addr']] = $iface['Inet Addr'];
	return $arrResult;
    }

    function sendConfigOpenVPN($client)
    {
	//Attention: needs checking file
	if(isset($client) && $client !== '')
	{
	    file_put_contents("/tmp/openvpn_$client.ovpn",$this->generateConfigOpenVPN($client));
	    `sudo /usr/sbin/elastix-helper openvpn_server --mailto --client $client`;
	    unlink("/tmp/openvpn_$client.ovpn");
	}
    }

    function generateConfigOpenVPN($client)
    {
	$pNet = new paloNetwork();
	$pNetConfig = $pNet->obtener_configuracion_red();
	$hostname = $pNetConfig['host'];
	$settings = $this->loadSettings();
	$result = "client\n";
	$result.= "dev tun\n";
	$result.= ( ($settings["pa_proto"] == "udp") ? "proto udp\n" : "proto tcp-client\n");

	if($settings["use_hostname"] == "on")
	    $result.= "remote ".$hostname;
	else
	{
	    if($settings["pa_local"]!='0.0.0.0')
		$result.= "remote ".$hostname;
	    else
	    {
		$out=NULL;
		exec("curl -s icanhazip.com --max-time 3; echo",$out);
		if(preg_match('/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\z/', $out[0]))
		    $result.= "remote ".$out[0];
		else
		    $result.= "remote ".$hostname;
	    }
	}
	    
	    
	$result.= " ".$settings["pa_port"]."\n";
	$result.= "resolv-retry infinite\n";
	$result.= "nobind\n";
	$result.= "persist-key\n";
	$result.= "persist-tun\n";
	$result.= ($settings["pa_compr"] == "on" ? "comp-lzo\n" : "");
	$result.= "verb 1\n";
	$result.= "ns-cert-type server\n";
	$ret = NULL;
	$ret = split("\n", `sudo /usr/sbin/elastix-helper openvpn_server --dumpuserconfig --client $client`);
	foreach($ret as $n => $rstr)
	    $result.="$rstr\n";
	
	return $result;
    }

    function loadSettings($param=FALSE)
    {

	$DBSettings_tmp = $this->_DB->fetchTable("SELECT * FROM settings",true);
	$arrValues = $DBSettings_tmp[0];

	foreach ($arrValues as &$value)
	    $value= ($value == '%EMPTY%' ? "":$value);
	
	if($arrValues['static_routes'] != '')
	    $arrValues['static_routes'] = str_replace(",","\n",$arrValues['static_routes']);
	return $arrValues;
    }
    
    function saveMailTemplateOpenVPN($params)
    {
	$arrParameters = array ($params['subject'],$params['body'],$params['realname'],$params['from_mail']);
	
	$request = "UPDATE mail SET subject=?,body=?,realname=?,from_mail=?";
	if ($this->_DB->genQuery($request,$arrParameters) != 0) return true;
	else return false;
    }
    
    function loadMailTemplate($param)
    {
	$DBSettings_tmp = $this->_DB->fetchTable("SELECT * FROM mail LIMIT 1",true);
	$arrValues = $DBSettings_tmp[0];

	switch ($param)
	{
	    case "SUBJECT":
		return $arrValues['subject'];
		break;
	    case "BODY":
		return $arrValues['body'];
		break;
	    case "REALNAME":
		return $arrValues['realname'];
		break;
	    case "FROM_MAIL":
		return $arrValues['from_mail'];
		break;
	}
    }
    
    function checkInstall($namePackage)
    {
    	global $arrLang;
    	$this->errMsg = '';
	exec("rpm -q $namePackage",$respuesta,$retorno);
	if($retorno == 0){
	    return TRUE;
	}
	else{
	    return FALSE;
	}
    }
    
    function checkTables()
    {
	global $arrLang;
    	$this->errMsg = '';

	if($this->getStatusServiceOpenVPN()=="active")
	{
	    global $arrLang;
    	    $this->errMsg = '';
	    $settings=$this->loadSettings();
	    if($settings['enable_nat'] == 'on')
	    {	
		$sComando = '/usr/bin/elastix-helper openvpn_server --checktables --server_network '.$settings['pa_server_network'].' 2>&1';
    		$output = $ret = NULL;
    		exec($sComando, $output, $ret);
    		if ($ret != 0 && $ret!=2) {
    		    return FALSE;
		}
	    }
	    return TRUE;
	}
	else return TRUE;
	
    }
    function validate($type,$val)
    {
	$return = false;
	switch ($type)
	{
	    case 'ip/mask':
		if(preg_match("/^([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})\/([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})$/", $val, $arrReg)) {
		    if(($arrReg[1]<=255) and ($arrReg[1]>0) and ($arrReg[2]<=255) and ($arrReg[2]>=0) and ($arrReg[3]<=255) and ($arrReg[3]>=0) and ($arrReg[4]<=255) and ($arrReg[4]>=0) 
			and ($arrReg[5]<=255) and ($arrReg[5]>0) and ($arrReg[6]<=255) and ($arrReg[6]>=0) and ($arrReg[7]<=255) and ($arrReg[7]>=0) and ($arrReg[8]<=255) and ($arrReg[8]>=0)) {
			    $return = true;
		    } 
                }
		break;
	    case 'ip/mask_array':
		$return = true;
		foreach($val as $v)
		{
		    if(preg_match("/^([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})\/([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})\.([[:digit:]]{1,3})$/", $v, $arrReg)) {
			if(($arrReg[1]<=255) and ($arrReg[1]>0) and ($arrReg[2]<=255) and ($arrReg[2]>=0) and ($arrReg[3]<=255) and ($arrReg[3]>=0) and ($arrReg[4]<=255) and ($arrReg[4]>=0) 
			        and ($arrReg[5]<=255) and ($arrReg[5]>0) and ($arrReg[6]<=255) and ($arrReg[6]>=0) and ($arrReg[7]<=255) and ($arrReg[7]>=0) and ($arrReg[8]<=255) and ($arrReg[8]>=0)) {
			        $return = true;
		    	} 
            	    }
            	    else
		    {
		        return false;
		    }
            	}
		break;
	    case 'alphanum':
	        if(eregi("^([[:alnum:]._-])+$", $val)) $return = true;
		else $return = false;
		break;
                                                       
	}
	return $return;
    }
    function ip_in_range($ip, $range) {
	if (strpos($range, '/') !== false) {
	// $range is in IP/NETMASK format
	    list($range, $netmask) = explode('/', $range, 2);
	    if (strpos($netmask, '.') !== false) {
    		// $netmask is a 255.255.0.0 format
    		$netmask = str_replace('*', '0', $netmask);
    		$netmask_dec = ip2long($netmask);
    		return ( (ip2long($ip) & $netmask_dec) == (ip2long($range) & $netmask_dec) );
	    } else {
    		// $netmask is a CIDR size block
    		// fix the range argument
    		$x = explode('.', $range);
    		while(count($x)<4) $x[] = '0';
    		list($a,$b,$c,$d) = $x;
    		$range = sprintf("%u.%u.%u.%u", empty($a)?'0':$a, empty($b)?'0':$b,empty($c)?'0':$c,empty($d)?'0':$d);
    		$range_dec = ip2long($range);
    		$ip_dec = ip2long($ip);

    		$wildcard_dec = pow(2, (32-$netmask)) - 1;
    		$netmask_dec = ~ $wildcard_dec;
		return (($ip_dec & $netmask_dec) == ($range_dec & $netmask_dec));
	    }
	} else {
	// range might be 255.255.*.* or 1.2.3.0-1.2.3.255
	    if (strpos($range, '*') !==false) { // a.b.*.* format
    		// Just convert to A-B format by setting * to 0 for A and 255 for B
    		$lower = str_replace('*', '0', $range);
    		$upper = str_replace('*', '255', $range);
    		$range = "$lower-$upper";
	    }

	    if (strpos($range, '-')!==false) { // A-B format
    		list($lower, $upper) = explode('-', $range, 2);
    		$lower_dec = (float)sprintf("%u",ip2long($lower));
    		$upper_dec = (float)sprintf("%u",ip2long($upper));
    		$ip_dec = (float)sprintf("%u",ip2long($ip));
    		return ( ($ip_dec>=$lower_dec) && ($ip_dec<=$upper_dec) );
	    }
	    return false;
	}
    }
    function getCountryListOpenVPN()
    {
	$country = array();
	$file_path = 'modules/openvpn_server/configs/country.lst';
	if(file_exists($file_path) && is_readable($file_path))
	{
	    $lines = file($file_path);
	    foreach ($lines as $line_num => $line) {
		$line_array = split(" ",$line);
		$short_name = $line_array[1];
		$full_name = "";
		for($i=4;$i<count($line_array);$i++)
		    $full_name .= trim($line_array[$i])." ";
		$country[$short_name] = $full_name;
	    }
	}
	return $country;
    }
}
?>
